@@ -1,39 +1,31 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
-
-# Copyright (c) 2014, 2015
-
-# Author(s):
-
-# Panu Lahtinen <panu.lahtinen@fmi.fi>
-# Hróbjartur Thorsteinsson <thorsteinssonh@gmail.com>
-
+#
+# Copyright (c) 2014-2020 Trollsift Developers
+#
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
-
-
-"""Parser class
-"""
+"""Main parsing and formatting functionality."""
 
 import re
 import datetime as dt
 import random
 import string
+from functools import lru_cache
 
 
 class Parser(object):
-    """Parser class
-    """
+    """Class-based interface to parsing and formatting functionality."""
 
     def __init__(self, fmt):
         self.fmt = fmt
@@ -141,7 +133,16 @@ formatter = StringFormatter()
 spec_regexes = {
     'c': r'.',
     'd': r'[-+]?\d',
-    'f': r'[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?',
+    'f': {
+        # Naive fixed point format specifier (e.g. {foo:f})
+        'naive': r'[-+]?(\d+(\.\d*)?|\.\d+)([eE][-+]?\d+)?',
+        # Fixed point format specifier including width and precision
+        # (e.g. {foo:4.2f}). The lookahead (?=.{width}) makes sure that the
+        # subsequent pattern is only matched if the string has the required
+        # (minimum) width.
+        'precision': r'(?=.{{{width}}})([-+]?([\d ]+(\.\d{{{decimals}}})+|\.\d{{{decimals}}})([eE][-+]?\d+)?)'
+
+    },
     'i': r'[-+]?(0[xX][\dA-Fa-f]+|0[0-7]*|\d+)',
     'o': r'[-+]?[0-7]',
     's': r'\S',
@@ -152,6 +153,7 @@ spec_regexes['E'] = spec_regexes['f']
 spec_regexes['g'] = spec_regexes['f']
 spec_regexes['X'] = spec_regexes['x']
 allow_multiple = ['c', 'd', 'o', 's', 'x', 'X']
+fixed_point_types = ['f', 'e', 'E', 'g']
 # format_spec ::=  [[fill]align][sign][#][0][width][,][.precision][type]
 # https://docs.python.org/3.4/library/string.html#format-specification-mini-language
 fmt_spec_regex = re.compile(
@@ -159,6 +161,26 @@ fmt_spec_regex = re.compile(
     r'(?P<comma>,)?(?P<precision>.\d+)?(?P<type>[bcdeEfFgGnosxX%])')
 
 
+def _get_fixed_point_regex(regex_dict, width, precision):
+    """Get regular expression for fixed point numbers.
+
+    Args:
+        width: Total width of the string representation.
+        precision: Number of decimals.
+    """
+    if width or precision:
+        if precision is None:
+            precision = '0,'
+        else:
+            precision = precision.strip('.')
+        if width is None:
+            width = '1,'
+        return regex_dict['precision'].format(
+            width=width, decimals=precision)
+    else:
+        return regex_dict['naive']
+
+
 class RegexFormatter(string.Formatter):
     """String formatter that converts a format string to a regular expression.
     
@@ -188,7 +210,7 @@ class RegexFormatter(string.Formatter):
     # special string to mark a parameter not being specified
     UNPROVIDED_VALUE = '<trollsift unprovided value>'
     ESCAPE_CHARACTERS = ['\\'] + [x for x in string.punctuation if x not in '\\%']
-    ESCAPE_SETS = [(c, '\{}'.format(c)) for c in ESCAPE_CHARACTERS]
+    ESCAPE_SETS = [(c, '\\' + c) for c in ESCAPE_CHARACTERS]
 
     def __init__(self):
         # hold on to fields we've seen already so we can reuse their
@@ -196,6 +218,7 @@ class RegexFormatter(string.Formatter):
         self._cached_fields = {}
         super(RegexFormatter, self).__init__()
 
+    @lru_cache()
     def format(*args, **kwargs):
         try:
             # super() doesn't seem to work here
@@ -255,6 +278,7 @@ class RegexFormatter(string.Formatter):
         ftype = regex_dict['type']
         width = regex_dict['width']
         align = regex_dict['align']
+        precision = regex_dict['precision']
         # NOTE: does not properly handle `=` alignment
         if fill is None:
             if width is not None and width[0] == '0':
@@ -263,13 +287,19 @@ class RegexFormatter(string.Formatter):
                 fill = ' '
 
         char_type = spec_regexes[ftype]
+        if ftype in fixed_point_types:
+            char_type = _get_fixed_point_regex(
+                char_type,
+                width=width,
+                precision=precision
+            )
         if ftype == 's' and align and align.endswith('='):
             raise ValueError("Invalid format specification: '{}'".format(format_spec))
         final_regex = char_type
         if ftype in allow_multiple and (not width or width == '0'):
             final_regex += r'*?'
         elif width and width != '0':
-            if not fill:
+            if not fill and ftype not in fixed_point_types:
                 # we know we have exactly this many characters
                 final_regex += r'{{{}}}'.format(int(width))
             elif fill:
@@ -346,28 +376,15 @@ def _get_number_from_fmt(fmt):
 
 def _convert(convdef, stri):
     """Convert the string *stri* to the given conversion definition *convdef*."""
+    is_fixed_point = any([ftype in convdef for ftype in fixed_point_types])
     if '%' in convdef:
         result = dt.datetime.strptime(stri, convdef)
-    elif 'd' in convdef or 's' in convdef:
-        regex_match = fmt_spec_regex.match(convdef)
-        match_dict = regex_match.groupdict() if regex_match else {}
-        align = match_dict.get('align')
-        pad = match_dict.get('fill')
-        if align:
-            # align character is the last one
-            align = align[-1]
-        if align and align in '<>^' and not pad:
-            pad = ' '
-
-        if align == '>':
-            stri = stri.lstrip(pad)
-        elif align == '<':
-            stri = stri.rstrip(pad)
-        elif align == '^':
-            stri = stri.strip(pad)
-
+    elif 'd' in convdef or 's' in convdef or is_fixed_point:
+        stri = _strip_padding(convdef, stri)
         if 'd' in convdef:
             result = int(stri)
+        elif is_fixed_point:
+            result = float(stri)
         else:
             result = stri
     else:
@@ -375,6 +392,31 @@ def _convert(convdef, stri):
     return result
 
 
+def _strip_padding(convdef, stri):
+    """Strip padding from the given string.
+
+    Args:
+        convdef: Conversion definition (indicates the padding)
+        stri: String to be modified
+    """
+    regex_match = fmt_spec_regex.match(convdef)
+    match_dict = regex_match.groupdict() if regex_match else {}
+    align = match_dict.get('align')
+    pad = match_dict.get('fill')
+    if align:
+        # align character is the last one
+        align = align[-1]
+    if align and align in '<>^' and not pad:
+        pad = ' '
+    if align == '>':
+        stri = stri.lstrip(pad)
+    elif align == '<':
+        stri = stri.rstrip(pad)
+    elif align == '^':
+        stri = stri.strip(pad)
+    return stri
+
+@lru_cache()
 def get_convert_dict(fmt):
     """Retrieve parse definition from the format string `fmt`."""
     convdef = {}
@@ -588,3 +630,14 @@ def is_one2one(fmt):
             return False
     # all checks passed, so just return True
     return True
+
+
+def purge():
+    """Clear internal caches.
+
+    Not needed normally, but can be used to force cache clear when memory
+    is very limited.
+
+    """
+    regex_formatter.format.cache_clear()
+    get_convert_dict.cache_clear()
